#ifndef INC_GTTEXTAREA_H
#define INC_GTTEXTAREA_H

#include "gtwidget.h"

/************************************************************************\
*
* GtTextArea class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Attributes for text */
#define GTTA_PLAIN			0x0000
#define GTTA_BOLD				0x0001
#define GTTA_UNDERLINE	0x0002
#define GTTA_REVERSE		0x0004
#define GTTA_MARKED			0x0080 /* Internal use */

class GtTextArea : public GtWidget
{
public:
		/* Standard */
	GtTextArea(void);
	void Destroy(void);
	sbool Init(char *n,GtManager *p,GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtTextArea"); }
	sbool IsSubClassOf(char *cn);
		/* Window management */
	void RefreshRect(int xp,int yp,int xs,int ys);
		/* Values */
	void SetEMask(ulong em);
	void GetColsRows(int *cs,int *rs) { *cs=ccols; *rs=crows; }
	void SetAttr(uword a) { cattr=a; }
	uword GetAttr(void) { return(cattr); }
		/* Text manipulation */
	void PutCh(int xp,int yp,ubyte c);
	void PutCh(int xp,int yp,ubyte c,uword a);
	void GetCh(int xp,int yp,ubyte *c);
	void GetCh(int xp,int yp,ubyte *c,uword *a);
	int PutStr(int xp,int yp,char *str,int len);
	int PutStr(int xp,int yp,char *str,int len,uword a);
	void Fill(int xp,int yp,int xs,int ys,ubyte c);
	void Fill(int xp,int yp,int xs,int ys,ubyte c,uword a);
	void ScrollRegion(int s1,int s2);
	void Scroll(sbool up);
	void InsertLine(int pos);
	void DeleteLine(int pos);
		/* Internal methods */
	void SetPosSize(word xp,word yp,word xs,word ys);
protected:
		/* Internal methods */
	sbool Draw(XEvent *xe);
	sbool Mouse(XEvent *xe);
private:
	void DrawLinePart(int xp,int yp,int y,int s,int e,uword a);
	void Mark(sbool on);
	void AllocMark(void);
	static int ConvMark(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
	static int DoneMark(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
protected:
		/* Instance variables */
	XRectangle rect;
	Time stime;
	ubyte **txt;
	uword **attr;
	uword cattr;
	word toff,tw,th;
	word trows,tcols,crows,ccols,mrows,mcols;
	word msxp,msyp,mexp,meyp,mxp,myp;
	word sr1,sr2;
	ubyte ftype,ibw,bdrw,ibdrw;
	sbool press;
};

extern GtTextArea *GtMkTextArea(char *n,GtManager *p,GtTag *tgs);
extern GtTextArea *GtMkTextArea(char *n,GtManager *p,ulong tag,...);

#endif /* INC_GTTEXTAREA_H */
