#include <proto.h>

#include "gttoggle.h"
#include "gtradiogroup.h"
#include "gtsymbol.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtToggle class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init toggle.
*
\************************************************************************/
GtToggle *GtMkToggle(char *n,GtManager *p,GtTag *tgs)
{
	GtToggle *obj;
	
	if(!(obj=new GtToggle()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtToggle object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init toggle.
*
\************************************************************************/
GtToggle *GtMkToggle(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtToggle *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkToggle(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty toggle.
*
\************************************************************************/
GtToggle::GtToggle(void)
{
	rgrp=NULL;
	ind=NULL;
	ton=FALSE;
	tbr=FALSE;
}

/************************************************************************\
*
* Setup toggle.
*
\************************************************************************/
sbool GtToggle::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	GtManager *pr;
	char *iname="togind";
	long ts;
	
	if(!GtButton::Init(n,p,tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_OnOff:
				if(t->d.val)
					ton=TRUE;
				else
					ton=FALSE;
				break;
			case GT_ToggleBorder:
				if(t->d.val)
					tbr=TRUE;
				else
					tbr=FALSE;
				break;
			case GT_SymbolName:
				iname=(char *)t->d.ptr;
				break;
		}
	}
	bflags&=~GTBF_REPEAT;
	for(pr=parent; pr; pr=pr->GetParent())
	{
		if(pr->IsSubClassOf("GtRadioGroup"))
		{
			rgrp=(GtRadioGroup *)pr;
			break;
		}
	}
	if(rgrp)
		ton=rgrp->SetReq(this,ton);
	if(!tbr && iname)
	{
		if(rgrp)
			ts=GTST_RADIO;
		else
			ts=GTST_TOGGLE;
		if(!(ind=GtMkSymbol(iname,this,GT_Symbol,ts,GT_Position,0,
																		GT_OnOff,ton,GT_DONE)))
			return(FALSE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Set toggle attributes.
*
\************************************************************************/
sbool GtToggle::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE,rt=FALSE;
	GtToggleCD cd;
	
	if(!GtButton::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_OnOff:
				if(t->d.val)
					ton=TRUE;
				else
					ton=FALSE;
				if(rgrp)
					ton=rgrp->SetReq(this,ton);
				ch=TRUE;
				break;
			case GT_RadioOnOff:
				if(t->d.val)
					ton=TRUE;
				else
					ton=FALSE;
				ch=TRUE;
				rt=TRUE;
				break;
		}
	}
	bflags&=~GTBF_REPEAT;
	if(ch)
	{
		if(ind)
		{
			if(!ind->SetV(GT_OnOff,ton,GT_DONE))
				return(FALSE);
		}
		if(tbr)
			this->Refresh();
		if(rt)
		{
			cd.button=0;
			cd.state=0;
			cd.multi=0;
			cd.on=ton;
			act.CallCB(this,&cd);
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Get toggle attributes.
*
\************************************************************************/
void GtToggle::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtButton::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_OnOff:
				*ptr=ton;
				break;
			case GT_ToggleBorder:
				*ptr=tbr;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtToggle::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtToggle")==0)
		return(TRUE);
	else
		return(GtButton::IsSubClassOf(cn));
}

/************************************************************************\
*
* Keep childrean to the right of indicator.
*
\************************************************************************/
sbool GtToggle::AddChild(GtWidget *obj,ulong pos)
{
	sbool ret=FALSE;

	if(ind)
		ret=GtButton::AddChild(obj,(pos>0 ? pos : 1));
	else
		ret=GtButton::AddChild(obj,pos);
	if(ret && !ind && obj->IsSubClassOf("GtSymbol"))
	{
		ind=(GtSymbol *)obj;
		ind->SetV(GT_OnOff,ton,GT_DONE);
	}
	return(ret);
}

/************************************************************************\
*
* Check for activate toggle notify.
*
\************************************************************************/
void GtToggle::Notify(ulong tag)
{
	GtToggleCD cd;
	sbool tmp;
	
	if(tag!=GT_Activate)
	{
		GtButton::Notify(tag);
		return;
	}
	tmp=ton;
	if(rgrp)
		ton=rgrp->ChangeReq(this,ton);
	else
		ton=!ton;
	if(!rgrp || (tmp!=ton))
	{
		if(ind)
			ind->SetV(GT_OnOff,ton,GT_DONE);
		if(tbr)
			this->Refresh();
	}
	cd.button=0;
	cd.state=0;
	cd.multi=0;
	cd.on=ton;
	if(!rgrp || (tmp!=ton))
		act.CallCB(this,&cd);
}

/************************************************************************\
*
* Draw toggled border.
*
\************************************************************************/
sbool GtToggle::Draw(XEvent *xe)
{
	if(tbr)
	{
		if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
			return(TRUE);
		if(flags & GTWF_DISABLED)
			gtdis->DrawRectangle(gtwin,0,0,xsize-1,ysize-1,TRUE);
		if(ton)
		{
			ftype=GTF_IN;
			GtGroup::Draw(NULL);
		}
		else
		{
			ftype=GTF_OUT;
			GtGroup::Draw(NULL);
		}
	}
	else
		GtButton::Draw(xe);
	return(TRUE);
}

/************************************************************************\
*
* Arm toggle button.
*
\************************************************************************/
void GtToggle::Arm(ulong button,ulong state)
{
	GtToggleCD cd;
	
	cd.button=button;
	cd.state=state;
	cd.multi=0;
	cd.on=ton;
	arm.CallCB(this,&cd);
}

/************************************************************************\
*
* Activate toggle button.
*
\************************************************************************/
void GtToggle::Activate(ulong button,ulong state)
{
	GtToggleCD cd;
	sbool tmp;
	
	tmp=ton;
	if(rgrp)
		ton=rgrp->ChangeReq(this,ton);
	else
		ton=!ton;
	if(!rgrp || (tmp!=ton))
	{
		if(ind)
			ind->SetV(GT_OnOff,ton,GT_DONE);
		if(tbr)
			this->Refresh();
	}
	cd.button=button;
	cd.state=state;
	cd.multi=mclk;
	cd.on=ton;
	if(!rgrp || (tmp!=ton))
		act.CallCB(this,&cd);
	cd.multi=0;
	darm.CallCB(this,&cd);
}

/************************************************************************\
*
* Disarm toggle button.
*
\************************************************************************/
void GtToggle::DisArm(ulong button,ulong state)
{
	GtToggleCD cd;
	
	cd.button=button;
	cd.state=state;
	cd.multi=0;
	cd.on=ton;
	darm.CallCB(this,&cd);
}

